%function BP_HOC_Demo
% This file runs the three solvers ISAL1, SPGL1 and l1-Magic on instance
% number 148 of the SPEAR L1 test set, both without and with HOC active,
% to demonstrate the effect of HOC on the solution speed and quality that 
% can be achieved for Basis Pursuit problems.
% (All solvers are run with their respective default settings.)
%
% See also Figure 10 in the paper "Solving Basis Pursuit: Heuristic Opti-
% mality Check and Solver Comparison" by D. Lorenz, M. Pfetsch and 
% A. Tillmann, which illustrates results from this demo.
%
% NOTE: The original codes for SPGL1 and l1-Magic were obtained from their
% web sites, see www.cs.ubc.ca/~mpf/spgl1/ and www.l1-magic.org , resp.; 
% the respective original authors retain all copyrights for their programs,
% as per the associated licenses in those software packages. 
% The original codes were modified to incorporate HOC for the purpose of 
% the computational comparisons described in our paper and for this demo.
% (Note also that in l1-Magic, the linsolve-Options were not used because
% otherwise, the algorithm aborts/crashes without providing a solution.)
%
% More information about ISAL1 and HOC (as well as current code versions)
% can be found at wwwopt.mathematik.tu-darmstadt.de/spear
%
% -- 11/18/2013  A. Tillmann, TU Darmstadt, Germany
% =========================================================================
clc
fprintf('Basis Pursuit -- Demonstration of Heuristic Optimality Check (HOC)\n\n')

doISA = true;
doSPG = true;
dol1M = true;

root_dir = cd;

% Load problem data A,b and true solution x :
load spear_inst_148.mat
% display problems specifics:
fprintf('Instance specs:\n')
disp(specs)

% Demo run -- ISAL1 :
if doISA
    fprintf('*****     ISAL1 - without HOC     *****   ')
    % note: problem is scaled (to ||b||=1), iter. output is w.r.t. scaled prob.
    % restart is enabled by default - not improving things in this example...
    if(input('Enable automatic restart? y/n [y] ','s')=='n') reset = 0;else reset = 1;end
    cd([root_dir,'/ISAL1']);
    fprintf('Computing feas. violation and distance-to-optimality over ISAL1 iter.s ... ');
    [~,~,~,~,~,ISA_feasv,ISA_xdiffv] = ISAL1_noHOC_iterInfo(A,b,x,'reset',reset);
    fprintf('done.\nNow demonstrate ISAL1 (without HOC) without that additional overhead:\n');
    t0=tic;
    [xisa,~,~,~,it] = ISAL1_noHOC(A,b,'dspl',250,'reset',reset);
    results_ISA.t = toc(t0);                 % running time
    results_ISA.xdiff = norm(x-xisa);        % abs. error w.r.t. x
    results_ISA.feas = norm(A*xisa-b,'inf'); % inf.-norm feasibility err.
    results_ISA.iter = it;                   % ISAL1 iterations
    fprintf('\n*****     ISAL1 - with HOC        *****')
    t0=tic;
    [xisa,~,~,~,it] = ISAL1(A,b,'dspl',250,'reset',reset);
    results_ISA_HOC.t = toc(t0);                 % running time 
    results_ISA_HOC.xdiff = norm(x-xisa);        % abs. error w.r.t. x 
    results_ISA_HOC.feas = norm(A*xisa-b,'inf'); % inf.-norm feasibility err.
    results_ISA_HOC.iter = it;                   % ISAL1 iterations 
    fprintf('\nResult summary : ISAL1 w/o HOC  versus  ISAL1 w/ HOC\n')
    fprintf('--------------------------------------------------------\n')
    fprintf('|| x - x^* ||_2   :    %.3e  vs.  %.3e\n',results_ISA.xdiff,results_ISA_HOC.xdiff);
    fprintf('|| A x - b ||_inf :    %.3e  vs.  %.3e\n',results_ISA.feas,results_ISA_HOC.feas);
    fprintf('solution time [s] :        %4.2f  vs.      %4.2f\n',results_ISA.t,results_ISA_HOC.t);
    fprintf('iterations        :        %5d  vs.      %5d\n',results_ISA.iter,results_ISA_HOC.iter);
    
    %fprintf('\nPress any key to continue...\n')
    %pause
end

% Demo run -- SPGL1 :
if doSPG
    fprintf('*****     SPGL1 - without HOC     *****\n')
    cd([root_dir,'/SPGL1']);
    SPGoptions.verbosity = 0; % "quiet"
    % note: stated products w/ A or A^T do not cover HOC computations
    fprintf('Computing feas. violation and distance-to-optimality over SPGL1 iter.s ... ');
    [~,~,~,~,SPG_feasv,SPG_xdiffv] = spgl1_iterInfo(A,b,x,0,0,[],SPGoptions);
    fprintf('done.\nNow demonstrate SPGL1 (without HOC) without that additional overhead:\n');    
    SPGoptions.verbosity = 1; % "some output"
    t0=tic;
    [xspg,~,~,info] = spgl1(A,b,0,0,[],SPGoptions);
    results_SPG.t = toc(t0);                 % running time
    results_SPG.xdiff = norm(x-xspg);        % abs. error w.r.t. x
    results_SPG.feas = norm(A*xspg-b,'inf'); % inf.-norm feasibility err.
    results_SPG.iter = info.iter;            % SPGL1 iterations
    fprintf('\n*****     SPGL1 - with HOC        *****')
    t0=tic;
    [xspg,~,~,info] = spgl1_HOC(A,b,0,0,[],SPGoptions);
    results_SPG_HOC.t = toc(t0);                 % running time 
    results_SPG_HOC.xdiff = norm(x-xspg);        % abs. error w.r.t. x 
    results_SPG_HOC.feas = norm(A*xspg-b,'inf'); % inf.-norm feasibility err.
    results_SPG_HOC.iter = info.iter;            % SPGL1 iterations 
    fprintf('\nResult summary : SPGL1 w/o HOC  versus  SPGL1 w/ HOC\n')
    fprintf('--------------------------------------------------------\n')
    fprintf('|| x - x^* ||_2   :    %.3e  vs.  %.3e\n',results_SPG.xdiff,results_SPG_HOC.xdiff);
    fprintf('|| A x - b ||_inf :    %.3e  vs.  %.3e\n',results_SPG.feas,results_SPG_HOC.feas);
    fprintf('solution time [s] :        %4.2f  vs.      %4.2f\n',results_SPG.t,results_SPG_HOC.t);
    fprintf('iterations        :        %5d  vs.      %5d\n',results_SPG.iter,results_SPG_HOC.iter);
    
    %fprintf('\nPress any key to continue...\n')
    %pause
end

% Demo run -- l1-Magic :
if dol1M
    fprintf('*****   l1-MAGIC - without HOC    *****\n')     
    cd([root_dir,'/l1-MAGIC/Optimization']);
    fprintf('Computing feas. violation and distance-to-optimality over l1-MAGIC iter.s ... ');
    opts.SYM = true; opts.POSDEF = true;
    [temp,~] = linsolve(A*A',b,opts);
    x0 = A'*temp;
    [~,~,l1M_feasv,l1M_xdiffv] = l1eq_pd_iterInfo(x0,A,[],b,x);
    fprintf('done.\nNow demonstrate l1-MAGIC (without HOC) without that additional overhead:\n');    
    t0=tic;
    % starting point: projection of 0 onto {x|Ax=b}
    fprintf('Computing starting point ... ');
    opts.SYM = true; opts.POSDEF = true;
    [temp,~] = linsolve(A*A',b,opts);
    x0 = A'*temp;
    fprintf('done.\n');
    [xl1m,it] = l1eq_pd(x0,A,[],b);
    results_l1M.t = toc(t0);                 % running time
    results_l1M.xdiff = norm(x-xl1m);        % abs. error w.r.t. x
    results_l1M.feas = norm(A*xl1m-b,'inf'); % inf.-norm feasibility err.
    results_l1M.iter = it;                   % l1-MAGIC iterations
    fprintf('\n*****   l1-MAGIC - with HOC       *****\n')
    t0=tic;
    % starting point: projection of 0 onto {x|Ax=b}
    fprintf('Computing starting point ... ');
    opts.SYM = true; opts.POSDEF = true;
    [temp,~] = linsolve(A*A',b,opts);
    x0 = A'*temp;
    fprintf('done.\n');
    [xl1m,it] = l1eq_pd_HOC(x0,A,[],b);
    results_l1M_HOC.t = toc(t0);                 % running time 
    results_l1M_HOC.xdiff = norm(x-xl1m);        % abs. error w.r.t. x 
    results_l1M_HOC.feas = norm(A*xl1m-b,'inf'); % inf.-norm feasibility err.
    results_l1M_HOC.iter = it;                   % l1-MAGIC iterations 
    fprintf('\nResult summary : l1-MAGIC w/o HOC  versus  l1-MAGIC w/ HOC\n')
    fprintf('--------------------------------------------------------\n')
    fprintf('|| x - x^* ||_2   :    %.3e  vs.  %.3e\n',results_l1M.xdiff,results_l1M_HOC.xdiff);
    fprintf('|| A x - b ||_inf :    %.3e  vs.  %.3e\n',results_l1M.feas,results_l1M_HOC.feas);
    fprintf('solution time [s] :        %4.2f  vs.      %4.2f\n',results_l1M.t,results_l1M_HOC.t);
    fprintf('iterations        :        %5d  vs.      %5d\n',results_l1M.iter,results_l1M_HOC.iter);
end

cd(root_dir);

%save('results_BP_HOC_Demo.mat','results_ISA','results_ISA_HOC','results_SPG','results_SPG_HOC','results_l1M','results_l1M_HOC');

% optionally plot feasibility violation and distance-to-optimum curves as
% in Figure 10 of the above-mentioned paper:
if( input('Create plot(s)? y/n [n] ','s')=='y') createPlots = 1;else createPlots = 0;end
if createPlots
    % create plots :   
    yticks = [1e-12,1e-6,1e0,1e6];
    fontsz = 16;
    
    if doISA
        % ISAL1-plot:
        figure1 = figure('XVisual',...
            '0x23 (TrueColor, depth 24, RGB mask 0xff0000 0xff00 0x00ff)');    
        h1=semilogy(0:length(ISA_feasv)-1,ISA_feasv,'-b','LineWidth',2);      
        hold;
        h2=semilogy(0:length(ISA_xdiffv)-1,ISA_xdiffv,'-r','LineWidth',2);          
        h3=line([results_ISA_HOC.iter;results_ISA_HOC.iter],[1e-20;1e11],'Color','k','LineStyle','--','LineWidth',2); 
        h5=semilogy(results_ISA_HOC.iter,results_ISA_HOC.feas,'ob','MarkerSize',8,'LineWidth',2);
        h6=semilogy(results_ISA_HOC.iter,results_ISA_HOC.xdiff,'or','MarkerSize',8,'LineWidth',2); 
        h4=line([0;length(ISA_feasv)-1],[1e-6;1e-6],'Color','k','LineStyle','-.','LineWidth',1,'DisplayName','1e-6');
        hx=line([0;length(ISA_feasv)-1],[1e-1;1e-1],'Color','k','LineStyle','-.','LineWidth',1,'DisplayName','1e-1');
        ylim([1e-12 1e6]);
        xlim([0 length(ISA_feasv)-1]);
        xlabel('Iteration k','FontSize',fontsz);
        set(gca,'XTick',[0,results_ISA_HOC.iter,length(ISA_feasv)-1]);
        %ylabel(sprintf('Feasibility Violation (blue) /\nDistance to Optimum (red)'),'FontSize',20);
        set(gca,'YTick',yticks)
        set(gca,'FontSize',fontsz)
        l=legend(gca,'${\left\Vert A{x}^{k}-\,~b\,\right\Vert}_{\infty}$','${\left\Vert {x}^{k}-~\,x^*\right\Vert}_{2}$','HOC success','${\left\Vert A\bar{x}-\,~b\,\right\Vert}_{\infty}$','${\left\Vert\bar{x}-~\,x^*\right\Vert}_{2}$');
        set(l,'Location','EastOutside','Interpreter','Latex');
        orient landscape
        saveas(gcf,'HOC_in_ISAL1','pdf');
    end
    
    if doSPG
        % SPGL1-plot:
        figure1 = figure('XVisual',...
            '0x23 (TrueColor, depth 24, RGB mask 0xff0000 0xff00 0x00ff)');    
        h1=semilogy(0:length(SPG_feasv)-1,SPG_feasv,'-b','LineWidth',2);      
        hold;
        h2=semilogy(0:length(SPG_xdiffv)-1,SPG_xdiffv,'-r','LineWidth',2);          
        h3=line([results_SPG_HOC.iter;results_SPG_HOC.iter],[1e-20;1e11],'Color','k','LineStyle','--','LineWidth',2);  
        h5=semilogy(results_SPG_HOC.iter,results_SPG_HOC.feas,'ob','MarkerSize',8,'LineWidth',2);
        h6=semilogy(results_SPG_HOC.iter,results_SPG_HOC.xdiff,'or','MarkerSize',8,'LineWidth',2);
        h4=line([0;length(SPG_feasv)-1],[1e-6;1e-6],'Color','k','LineStyle','--','LineWidth',1,'DisplayName','1e-6');
        hx=line([0;length(SPG_feasv)-1],[1e-1;1e-1],'Color','k','LineStyle','--','LineWidth',1,'DisplayName','1e-1');
        ylim([1e-12 1e6]);
        xlim([0 length(SPG_feasv)-1]);
        xlabel('Iteration k','FontSize',fontsz);
        set(gca,'XTick',[0,results_SPG_HOC.iter,length(SPG_feasv)-1]);
        %ylabel(sprintf('Feasibility Violation (blue) /\nDistance to Optimum (red)'),'FontSize',20);
        set(gca,'YTick',yticks)
        set(gca,'FontSize',fontsz)
        l=legend(gca,'${\left\Vert A{x}^{k}-\,~b\,\right\Vert}_{\infty}$','${\left\Vert {x}^{k}-~\,x^*\right\Vert}_{2}$','HOC success','${\left\Vert A\bar{x}-\,~b\,\right\Vert}_{\infty}$','${\left\Vert\bar{x}-~\,x^*\right\Vert}_{2}$');
        set(l,'Location','EastOutside','Interpreter','Latex');
        orient landscape
        saveas(gcf,'HOC_in_SPGL1','pdf');
    end
    
    if dol1M
        % l1-MAGIC-plot:
        figure1 = figure('XVisual',...
            '0x23 (TrueColor, depth 24, RGB mask 0xff0000 0xff00 0x00ff)');    
        h1=semilogy(0:length(l1M_feasv)-1,l1M_feasv,'-b','LineWidth',2);      
        hold;
        h2=semilogy(0:length(l1M_xdiffv)-1,l1M_xdiffv,'-r','LineWidth',2);          
        h3=line([results_l1M_HOC.iter;results_l1M_HOC.iter],[1e-20;1e11],'Color','k','LineStyle','--','LineWidth',2);  
        h5=semilogy(results_l1M_HOC.iter,results_l1M_HOC.feas,'ob','MarkerSize',8,'LineWidth',2);
        h6=semilogy(results_l1M_HOC.iter,results_l1M_HOC.xdiff,'or','MarkerSize',8,'LineWidth',2);
        h4=line([0;length(l1M_feasv)-1],[1e-6;1e-6],'Color','k','LineStyle','--','LineWidth',1,'DisplayName','1e-6');
        hx=line([0;length(l1M_feasv)-1],[1e-1;1e-1],'Color','k','LineStyle','--','LineWidth',1,'DisplayName','1e-1');
        ylim([1e-12 1e6]);
        xlim([0 length(l1M_feasv)-1]);
        xlabel('Iteration k','FontSize',fontsz);
        set(gca,'XTick',[0,results_l1M_HOC.iter,length(l1M_feasv)-1]);
        %ylabel(sprintf('Feasibility Violation (blue) /\nDistance to Optimum (red)'),'FontSize',20);
        set(gca,'YTick',yticks)
        set(gca,'FontSize',fontsz)
        l=legend(gca,'${\left\Vert A{x}^{k}-\,~b\,\right\Vert}_{\infty}$','${\left\Vert {x}^{k}-~\,x^*\right\Vert}_{2}$','HOC success','${\left\Vert A\bar{x}-\,~b\,\right\Vert}_{\infty}$','${\left\Vert\bar{x}-~\,x^*\right\Vert}_{2}$');
        set(l,'Location','EastOutside','Interpreter','Latex');
        orient landscape
        saveas(gcf,'HOC_in_l1-MAGIC','pdf');
    end
end

% Grayscale plots w/ fewer annotations --> Figure 10 of the paper: 
%papersz = [12 9];
%paperpos = [0 0 12.5 9];
% ISAL1-plot:
% figure;
% h1=semilogy(0:length(ISA_feasv)-1,ISA_feasv,'-','Color',[.6 .6 .6],'LineWidth',2);
% hold;
% h2=semilogy(0:length(ISA_xdiffv)-1,ISA_xdiffv,'-k','LineWidth',2);
% h3=line([results_ISA_HOC.iter;results_ISA_HOC.iter],[1e-20;1e11],'Color','k','LineStyle','--','LineWidth',2);
% h5=semilogy(results_ISA_HOC.iter,results_ISA_HOC.feas,'o','Color',[.6 .6 .6],'MarkerSize',8,'LineWidth',2);
% h6=semilogy(results_ISA_HOC.iter,results_ISA_HOC.xdiff,'ok','MarkerSize',8,'LineWidth',2);
% h4=line([0;length(ISA_feasv)-1],[1e-6;1e-6],'Color','k','LineStyle','--','LineWidth',1,'DisplayName','1e-6');
% hx=line([0;length(ISA_feasv)-1],[1e-1;1e-1],'Color','k','LineStyle','--','LineWidth',1,'DisplayName','1e-1');
% ylim([1e-12 1e6]);
% xlim([0 length(ISA_feasv)-1]);
% xlabel('Iteration k','FontSize',fontsz);
% set(gca,'XTick',[0,results_ISA_HOC.iter,length(ISA_feasv)-1]);
% set(gca,'YTick',yticks)
% set(gca,'FontSize',fontsz)
% set(gcf,'PaperPosition', paperpos);
% set(gcf','Papersize', papersz);
% saveas(gcf,'HOC_example_ISAL1','pdf');
% % SPGL1-plot:
% figure;
% h1=semilogy(0:length(SPG_feasv)-1,SPG_feasv,'-','Color',[.6 .6 .6],'LineWidth',2);
% hold;
% h2=semilogy(0:length(SPG_xdiffv)-1,SPG_xdiffv,'-k','LineWidth',2);
% h3=line([results_SPG_HOC.iter;results_SPG_HOC.iter],[1e-20;1e11],'Color','k','LineStyle','--','LineWidth',2);
% h5=semilogy(results_SPG_HOC.iter,results_SPG_HOC.feas,'o','Color',[.6 .6 .6],'MarkerSize',8,'LineWidth',2);
% h6=semilogy(results_SPG_HOC.iter,results_SPG_HOC.xdiff,'ok','MarkerSize',8,'LineWidth',2);
% h4=line([0;length(SPG_feasv)-1],[1e-6;1e-6],'Color','k','LineStyle','--','LineWidth',1,'DisplayName','1e-6');
% hx=line([0;length(SPG_feasv)-1],[1e-1;1e-1],'Color','k','LineStyle','--','LineWidth',1,'DisplayName','1e-1');
% ylim([1e-12 1e6]);
% xlim([0 length(SPG_feasv)-1]);
% xlabel('Iteration k','FontSize',fontsz);
% set(gca,'XTick',[0,results_SPG_HOC.iter,length(SPG_feasv)-1]);
% set(gca,'YTick',yticks)
% set(gca,'FontSize',fontsz)
% set(gcf,'PaperPosition', paperpos);
% set(gcf','Papersize', papersz);
% saveas(gcf,'HOC_example_SPGL1','pdf');
% % l1-MAGIC-plot:
% figure;
% h1=semilogy(0:length(l1M_feasv)-1,l1M_feasv,'-','Color',[.6 .6 .6],'LineWidth',2);
% hold;
% h2=semilogy(0:length(l1M_xdiffv)-1,l1M_xdiffv,'-k','LineWidth',2);
% h3=line([results_l1M_HOC.iter;results_l1M_HOC.iter],[1e-20;1e11],'Color','k','LineStyle','--','LineWidth',2);
% h5=semilogy(results_l1M_HOC.iter,results_l1M_HOC.feas,'o','Color',[.6 .6 .6],'MarkerSize',8,'LineWidth',2);
% h6=semilogy(results_l1M_HOC.iter,results_l1M_HOC.xdiff,'ok','MarkerSize',8,'LineWidth',2);
% h4=line([0;length(l1M_feasv)-1],[1e-6;1e-6],'Color','k','LineStyle','--','LineWidth',1,'DisplayName','1e-6');
% hx=line([0;length(l1M_feasv)-1],[1e-1;1e-1],'Color','k','LineStyle','--','LineWidth',1,'DisplayName','1e-1');
% ylim([1e-12 1e6]);
% xlim([0 length(l1M_feasv)-1]);
% xlabel('Iteration k','FontSize',fontsz);
% set(gca,'XTick',[0,results_l1M_HOC.iter,length(l1M_feasv)-1]);
% set(gca,'YTick',yticks)
% set(gca,'FontSize',fontsz)
% set(gcf,'PaperPosition', paperpos);
% set(gcf','Papersize', papersz);
% saveas(gcf,'HOC_example_l1-MAGIC','pdf');