function [A,x,b] = readdata(matrixfile,vectorfile,matsp,vectsp,rhs)
% =========================================================================
%  FILE:           readdata.m
%  AUTHOR:         Andreas Tillmann, TU Darmstadt, Germany 
%  LAST MODIFIED:  06/21/2011
% =========================================================================
%  DESCRIPTION:    This function reads a matrix A and vector x from input 
%                  ASCII-files and creates a vector b:=A*x. Optionally, 
%                  the program can read only one of A and x (and then of
%                  course not compute b), or read the vector b instead of 
%                  x. All data is assumed to be real-valued.
%                  
%                  SPECIAL FILENAME STRUCTURE ASSUMED, WHICH CORRESPONDS TO
%                  THE NAMING CONVENTIONS USED FOR THE TESTSET FOR THE
%                  COMPUTATIONAL COMPARISON OF L1-SOLVERS PERFORMED IN
%                  D.Lorenz,M.Pfetsch,A.Tillmann-"Solving Basis Pursuit:
%                  Heuristic Optimality Check and Solver Comparison", 2011/12
%
%  INPUT:          matrixfile   Name of the file containing the matrix A.
%                               NOTE: If the filename contains the string
%                               'sparse' the matrix is assumed to be stored
%                               as a sparse matrix, i.e., in the format
%                                   m n       % matrix dimensions
%                                   nnz       % number of nonzero entries
%                                   i j val   % row, column, entry value,
%                               else the matrix is generally assumed to be
%                               stored explicitly/dense. Can be [] if only
%                               a vector shall be read.
%                  vectorfile   Name of the file containing the vector x.
%                               NOTE: Vectors are generally assumed to be
%                               stored in sparse format, i.e., 
%                                   n         % vector length
%                                   i val     % index, entry value.
%                               Can be [] if only a matrix shall be read.
%                  matsp        A binary argument specifying whether the
%                               matrix is given explicitly (matsp=0) or in
%                               sparse (ASCII) format (matsp=1). This argu-
%                               ment is needed only if the input matrixfile
%                               contains the matrix data in a way such that
%                               the defaults do not fit.
%                               [ optional; default is matsp=0 ]
%                  vectsp       A binary argument specifying whether the
%                               vector is given explicitly (vectsp=0) or in
%                               sparse (ASCII) format (vectsp=1). This ar-
%                               gument is needed only if the input vector-
%                               file contains the vector data in a way such
%                               that the defaults do not fit.
%                               [ optional; default is vectsp=1 ]
%                  rhs          A binary argument specifying whether the
%                               given vector file contains x (rhs=0) or 
%                               b=Ax (rhs=1). 
%                               [ optional; default is rhs=0 ]
%                  
%  OUTPUT:         A            The matrix read from file matrixfile;
%                               matrix format (full/sparse) depends on                               
%                               filename and the value of the parameter 
%                               matsp. If matrixfile was [], so will be A.
%                  x            The vector read from file vectorfile;
%                               vector format (full/sparse) depends on the
%                               value of the parameter vectsp.
%                               If vectorfile was [], so will be x and b.
%                               If rhs was 1, x will be [].
%                  b            Right hand side vector of the linear system
%                               Ax=b; either computed from input A and x,
%                               or given by input file instead of x (depen-
%                               ding on the value of parameter rhs). The
%                               vector format (full/sparse) depends on the
%                               values of the parameters matsp (and/or the 
%                               matrixfile name) and vectsp. If vectorfile
%                               or matrixfile was [], so will be b.
% =========================================================================
%error(nargchk(2, 5, nargin, 'struct')); % check number of arguments

% Set defaults, if needed:
if( nargin < 3 )
    matsp = 0;
end
if( nargin < 4 )
    vectsp = 1;
end
if( nargin < 5 )
    rhs = 0;
end

% Initialize A, x and b as empty matrices:
A = []; x = []; b = [];

% Read vector (if applicable):
if( ~isempty(vectorfile) ) 
    xs = importdata(vectorfile,' ',1);  
    if( ( rhs == 0 ) && ( vectsp == 1 ) )     % x, sparse
        x = spalloc(str2double(xs.textdata{1}),1,size(xs.data,1));
        x(xs.data(:,1)) = xs.data(:,2);
    elseif( ( rhs == 0 ) && ( vectsp == 0 ) ) % x, given explicitly
        x = xs.data;
    elseif( ( rhs == 1 ) && ( vectsp == 1 ) ) % b, sparse
        b = spalloc(str2double(xs.textdata{1}),1,size(xs.data,1));
        b(xs.data(:,1)) = xs.data(:,2);
    elseif( ( rhs == 1 ) && ( vectsp == 0 ) ) % b, given explicitly
        b = xs.data;
    end
    clear xs;
end

% Read matrix (if applicable):
if( ~isempty(matrixfile) )
    if( ( ~isempty(strfind(matrixfile,'sparse')) ) || ...
            ( matsp == 1 ) ) % A, sparse
        As = importdata(matrixfile,' ',2);
        %[m n] = strread(As.textdata{1},'%d %d'); % not needed
        A = sparse(As.data(:,1),As.data(:,2),As.data(:,3));
    else                     % A, given explicitly
        As = importdata(matrixfile,' ',1);
        A = As.data;        
    end
    clear As;
end

% Compute b=Ax (if applicable):
if( ( rhs == 0 ) && ( ~isempty(x) ) && ( ~isempty(A) ) )
    b = A*x;
end
