function x = CG(A,AAT,b,tol,maxit,x0,CGmode)
% function x = CG(A,AAT,b,tol,maxit,x0,CGmode)
% =========================================================================
%  FILE:           CG.m
%  AUTHOR:         Andreas M. Tillmann, TU Darmstadt, Germany
%                  tillmann@mathematik.tu-darmstadt.de
% =========================================================================
%  DESCRIPTION:    A (basic) CG implementation (without preconditioning),
%                  specialized to the solution of normal equations of the
%                  form A*A'*x = b.
%                  NOTE: The user has to supply consistent data; no error
%                  or consistency checks whatsoever are performed here!
%
%  INPUT:   A      The matrix yielding the CG system matrix A*A'
%                  (may be empty, if AAT is provided instead)
%           AAT    The matrix A*A' given explicitly
%                  (unused and hence may be empty, if A is provided)
%           b      The right hand side vector
%           tol    The CG convergence tolerance
%           maxit  The maximal number of CG iterations allowed
%           x0     The starting point (initial solution guess)
%           CGmode Optional indicator; value 0: use AAT (then required),
%                  other values: use only A (then required; AAT is not).
%
%  OUTPUT:  x      The (approximate) solution computed by the CG method
% =========================================================================
% $Revision: 206 $
% $Author: dirloren $
% $Date: 2011-07-13 14:55:24 +0200 (Mi, 13. Jul 2011) $
% =========================================================================
x = x0; % initial guess
if( nargin < 7 ) % CGmode not provided
    if( ~isempty(A) )
        CGmode = 1; % use A
    elseif( ~isempty(AAT) )
        CGmode = 0; % use AAT, also if both A and AAT are provided
    else
        disp('Need matrix A or AAT.');
        return;
    end
end

if( CGmode )             % A is given (use this for sparse A in particular)
    wtemp = A'*x;
    %w = A*wtemp;    % w = A*A'*x
    r = b - A*wtemp; % r = b - w;
    p = r;
    d = r'*p;
    for iter = 1:maxit
        if( norm(p,2) < tol ) % convergence w.r.t. accuracy tolerance
            break;            % (small change in x)
        end
        wtemp = A'*p;
        w = A*wtemp;
        alpha = d/(p'*w);
        x = x + alpha*p;
        r = r - alpha*w;
        w = r;
        if( (norm(w,2) < tol) && (norm(r,2) < tol) )
            break;  % convergence (small residual)
        end
        dlast = d;
        d = r'*w;
        p = w + (d/dlast)*p;
    end
else % ~CGmode                   % AAT=A*A' is given (as function argument)
    %w = AAT*x;
    r = b - AAT*x;   % r = b - w;
    p = r;
    d = r'*p;
    for iter = 1:maxit
        if( norm(p,2) < tol ) % convergence w.r.t. accuracy tolerance
            break;            % (small change in x)
        end
        w = AAT*p;
        alpha = d/(p'*w);
        x = x + alpha*p;
        r = r - alpha*w;
        w = r;
        if( (norm(w,2) < tol) && (norm(r,2) < tol) )
            break;  % convergence (small residual)
        end
        dlast = d;
        d = r'*w;
        p = w + (d/dlast)*p;
    end
end